# Overview

This interface layer handles the communication between IBM DB2 and Consumer
charms. The provider end of this interface provides the IBM DB2 service. The
consumer part requires the existence of a provider to function.

## Usage

### Provides

This interface layer will set the following states, as appropriate:

- `{relation_name}.connected`: The relation is established, but IBM DB2 may not
yet have provided any connection or service information.

- `{relation_name}.sshconfigured`: A consumer ssh key has been added to the
IBM DB2 `authorized_keys` file.

- `{relation_name}.ready`: IBM DB2 has established relation with the consumer
charm and the SSH key has been exchanged. DB2 has provided its connection
string information, and is ready to accept requests from the consumer.

- `{relation_name}.departed`: The relation has been removed. Any cleanup
related to the consumer charm should happen now on the IBM DB2 charm since the
consumer is going away.


### Requires

Consumers like WAS, Mobile First Server, etc require this interface to connect
to IBM DB2. This interface layer will set the following states, as appropriate:

- `{relation_name}.connected`: The consumer charm has been related to IBM DB2.
At this point, the consumer charm sends the SSH key to IBM DB2 charm.

    For example, the consumer charm can send the ssh key to IBM DB2 charm as follows:

    ```
    @when 'db.connected'
    @when_not 'db.sshconfigured'
    function configure_sshkeys_dbs() {
        SSH_PATH=/root/.ssh
        if [ ! -f  $SSH_PATH/id_rsa.pub ]; then
                juju-log "Setting up SSH keys."
                ssh-keygen -t rsa -f $SSH_PATH/id_rsa -N ''
        fi
        key="`cat $SSH_PATH/id_rsa.pub`"
        relation_call --state=db.connected set_ssh_keys "$key" || true
    }
    ```

-  `{relation_name}.ready`: IBM DB2 is ready to send connection information.
The consumer charm can access the IBM DB2 details from the interface using the
following methods:

   - get_db2_hostname() - host name of IBM DB2
   - get_db2_port() - port number of IBM DB2
   - get_db2_path() - the install location of IBM DB2
   - get_dbusername() - User name created for the consumer service to connect to DB2
   - get_dbuserpw() - Password for connecting to DB2
   - get_db2_instance_name() - Instance name created for the consumer service


- `{relation_name}.departed`: The relation has been removed. Any cleanup
related to IBM DB2 should happen now on the consumer charm since IBM DB2 is
going away.
