# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from charms.reactive import hook
from charms.reactive import RelationBase
from charms.reactive import scopes


class db2Requires(RelationBase):
    scope = scopes.GLOBAL
    auto_accessors = ['ssh_key']

    @hook('{requires:db2}-relation-joined')
    def joined(self):
        self.remove_state('{relation_name}.departed')
        self.set_state('{relation_name}.connected')

    @hook('{requires:db2}-relation-changed')
    def changed(self):
        self.remove_state('{relation_name}.departed')
        self.set_state('{relation_name}.connected')
        if str(self.get_remote('db2_port')) != "None":
            self.set_state('{relation_name}.ready')

    @hook('{requires:db2}-relation-departed')
    def departed(self):
        self.remove_state('{relation_name}.connected')
        self.remove_state('{relation_name}.ready')
        self.set_state('{relation_name}.departed')

    def get_db2_port(self):
        return self.get_remote('db2_port')

    def get_db2_path(self):
        return self.get_remote('db2_path')

    def get_db2_hostname(self):
        return self.get_remote('hostname')

    def get_dbusername(self):
        return self.get_remote('dbusername')

    def get_dbuserpw(self):
        return self.get_remote('dbuserpw')

    def get_db2_instance_name(self):
        return self.get_remote('db2_instance_name')

    def get_db2_dbnames(self):
        return self.get_remote('dbs_created')

    def set_ssh_keys(self, ssh_key):
        self.set_remote('ssh_key', ssh_key)

    def set_dbs(self, db_names):
        self.set_remote('db_names', db_names)
