# IBM base layer

The IBM base layer is a common starting point for IBM related charms. Use this
layer if you are building a layered charm with IBM software.

## Usage

This charm is a base layer and not meant to be built as a charm on its own.

To use this layer, simply include the following in the runtime charm's
layer.yaml file

```yaml
includes: ['layer:ibm-base']
```

## Configuration

**curl_url** - Location of the IBM product installation file(s). This should be a
URL that curl can use to download files. Multiple URLs should be separated
by a space.

**NOTE**: cryptographic verification is required and must be
specified as part of the URL query string with the key a valid hash
algorithms md5, sha256, or sha512, and the the checksum value itself
(`http://<url>?[md5|sha256|sha512]=<checksum>`).

For example:  

* `http://example.com/file.tgz?sha256=<sum>`
* `"sftp://example.com/file1.tgz?md5=<sum> ftp://example.com/file2.tgz?md5=<sum>"`

**curl_options** - The options passed to the 'curl' command when fetching files
from curl_url.

For example:  

* `"-u <user:password>"`

**license_accepted** - Some IBM charms require acceptance of a license before
installation can proceed. If required, setting this option to True indicates
that you have read and accepted the IBM terms and conditions found in the
license file referenced by the charm.

## States

**ibm-base.curl.resource.fetched** - When this state is set the IBM base layer has
downloaded and verified the resources configured by curl_url. If the charm
does not require any curl resources this state can be ignored.

**ibm-base.license.accepted** - When this state is set the user has signified
their acceptance of the license found in the charm. If the charm implements
the Juju 'terms' feature this state can be safely ignored.

## Contacts

IBM Juju Support Team <jujusupp@us.ibm.com>
