from charms.reactive import hook
from charms.reactive import RelationBase
from charms.reactive import scopes


class db2Provides(RelationBase):
    # Every unit connecting will get the same information
    scope = scopes.SERVICE
    # convenient way to provide accessor methods
    auto_accessors = ['db2_path', 'db2_port']

    @hook('{provides:db2}-relation-joined')
    def joined(self):
        conversation = self.conversation()
        conversation.remove_state('{relation_name}.departed')
        conversation.set_state('{relation_name}.connected')

    @hook('{provides:db2}-relation-changed')
    def changed(self):
        conversation = self.conversation()
        conversation.remove_state('{relation_name}.departed')
        conversation.set_state('{relation_name}.connected')
        if str(conversation.get_remote('ssh_key')) != "None":
            conversation.set_state('{relation_name}.ready')

    @hook('{provides:db2}-relation-departed')
    def departed(self):
        conversation = self.conversation()
        conversation.remove_state('{relation_name}.connected')
        conversation.remove_state('{relation_name}.ready')
        conversation.set_state('{relation_name}.departed')

    def dismiss(self, service):
        conversation = self.conversation(service)
        conversation.remove_state('{relation_name}.departed')

    def reset_states(self, service):
        conversation = self.conversation(service)
        conversation.remove_state('{relation_name}.connected')
        conversation.remove_state('{relation_name}.departed')
        conversation.remove_state('{relation_name}.ready')

    def set_db_details(self, service, db2_path, db2_port, hostname,
                       dbusername, dbuserpw, db2_instance_name, dbs_created):
        conversation = self.conversation(service)
        conversation.set_remote(data={
            'db2_ready': True,
            'db2_path': db2_path,
            'db2_port': db2_port,
            'hostname': hostname,
            'dbusername': dbusername,
            'dbuserpw': dbuserpw,
            'db2_instance_name': db2_instance_name,
            'dbs_created': dbs_created,
        })

    def set_sshconfigured(self, service):
        conversation = self.conversation(service)
        conversation.set_state('{relation_name}.sshconfigured')

    def dismiss_sshconfigured(self, service):
        conversation = self.conversation(service)
        conversation.remove_state('{relation_name}.sshconfigured')

    def get_sshkeys(self, service):
        conversation = self.conversation(service)
        return conversation.get_remote('ssh_key')

    def get_dbnames(self, service):
        conversation = self.conversation(service)
        return conversation.get_remote('db_names')

    def services(self):
        """
        Return a list of services requesting databases.
        """
        service = []
        for conversation in self.conversations():
            service.append(conversation.scope)
        return service
