#!/bin/bash
set -e

source charms.reactive.sh

ARCHITECTURE=`uname -m`
ARCHIVE_DIR=$CHARM_DIR/files/archives
DB2_ARCHIVE_DIR=`pwd`
DB2_USER=db2inst1
DB2_USER_PW=`pwgen -N 1 15`
DB2_DAS_USER=dasusr1
DB2_DAS_USER_PW=`pwgen -N 1 15`
DB2_FENCE_USER=db2fenc1
DB2_FENCE_USER_PW=`pwgen -N 1 15`
DB2_INSTALL_PATH=/opt/ibm/db2/V11.1
AUTH_KEY_FILE=/root/.ssh/authorized_keys
TEMP_KEY_FILE=key.txt

cfgusername=""

remoteunit=$cfgusername
cfgpasswd=""
dbs_created=""

isdbcreated=0
#Get the cfguser name based on the remote unit name
get_cfgusername()
{
	usr="usr"
	db="db"
	cfgusername=$1
	cfgusername=`echo $cfgusername | cut -d"/" -f1`
	cfgusername=`echo "$cfgusername" | sed -r 's/-//g' | sed -r 's/ibm//g'`
	cfgusername=`echo "$cfgusername" | awk '{print substr($0,0,5)}'`
	remoteunit=$cfgusername
	cfgusername="$cfgusername$usr"
	remoteunitdbname="$remoteunit$db"

}
# Check whether DB2 is installed
is_db2_installed()
{
	if [ -d $DB2_INSTALL_PATH/bin ]; then
		echo "True"
	else
		echo "False"
	fi
}


# Remove DB2, if installed
remove_software()
{
	db2_inst=`is_db2_installed`
	if  [ $db2_inst == "True" ]; then
		juju-log "IBM DB2: Removing IBM DB2 software."
		status-set maintenance "Removing IBM DB2"

		#finding the DB alias names to be dropped and dropping all Databases in that
		for queue1 in `su - $DB2_USER -c 'db2ilist'`;
		do
			juju-log "IBM DB2: Dropping the Databases for the instance $queue1"
			#if db2 is not running, we need to start it first, otherwise db2 drop command will fail
			if su - $queue1 -c 'ps -eaf|grep -i "\bdb2sysc 0\b" |grep -i '$queue1;
			then
				juju-log "IBM DB2: DB2 already started for the user $queue1"
			else
				su - $queue1 -c 'db2start'
				juju-log "IBM DB2: DB2 is started"
			fi

			for queue in `su - $queue1 -c 'db2 list db directory | grep "Database alias" | cut -d"=" -f2'`;
			do
				queue=`echo $queue | xargs`
				juju-log "IBM DB2: Dropping the DB $queue"
				su - $queue1 -c 'db2 drop database '$queue
				if [ $? -eq 0 ]; then
					juju-log "IBM DB2: Dropped the DB $queue"
				else
					juju-log "IBM DB2: Dropping the DB $queue failed"
					status-set blocked "Dropping the DB failed"
					exit 0
				fi
			done
			juju-log "IBM DB2: Stopping the DB for $queue1"

			if su - $queue1 -c 'ps -eaf|grep -i "\bdb2sysc 0\b" |grep -i '$queue1
			then
				su - $queue1 -c 'db2 force application all'
				su - $queue1 -c 'db2stop'
				if [ $? -eq 0 ]; then
					juju-log "IBM DB2: Stopped the DB for the instance $queue1"
				else
					juju-log "IBM DB2: Stopping the DB for $queue1 failed"
					status-set blocked "Stopping the DB for $queue1 failed"
					exit 0
				fi

			else
				juju-log "IBM DB2: DB2 is already stopped"
			fi
		done

		#drop the das
		cd $DB2_INSTALL_PATH/instance
		juju-log "IBM DB2: Dropping the DAS"
		./dasdrop
		if [ $? -eq 0 ]; then
			juju-log "IBM DB2: Dropped the DAS"
		else
			juju-log "IBM DB2: Dropping the DAS failed"
			status-set blocked "Dropping the DAS failed"
			exit 0
		fi

		#delete the instances
		su - $DB2_USER -c 'db2ilist'
		for queue1 in `su - $DB2_USER -c 'db2ilist'`;
		do
			juju-log "IBM DB2: Deleting the instance $queue1"
			$DB2_INSTALL_PATH/instance/db2idrop $queue1
			if [ $? -eq 0 ]; then
				juju-log "IBM DB2: Deleted  the instance $queue1"
			else
				juju-log "IBM DB2: Deleting the instance $queue1 failed"
				status-set blocked "Deleting the instance $queue1 failed"
				exit 0
			fi

		done

		#Delete the instance entries from /etc/services file
		juju-log "IBM DB2: Deleting the entries from /etc/services"
		sed -i '/db2c_/d' /etc/services
		if [ $? -eq 0 ]; then
			juju-log "IBM DB2: Deleted the entries from /etc/services"
		else
			juju-log "IBM DB2: Deletion of the entries from /etc/services failed"
			status-set blocked "Deletion of the entries from /etc/services failed"
			exit 0
		fi

		#uninstall
		juju-log "IBM DB2: Uninstall "
		cd $DB2_INSTALL_PATH/install
		set +e
		./db2_deinstall -a
		if [ $? -eq 0 ]; then
			juju-log "IBM DB2: Uninstall of db2 successful"
		else
			juju-log "IBM DB2: Uninstall of db2 failed"
			status-set blocked "IBM DB2 Uninstallation failed"
			exit 0
		fi
		set -e
		cd /opt/ibm
		rm -rf db2
		juju-log "IBM DB2: Finished Uninstalling......."
		status-set blocked "IBM DB2 Uninstalled"

	else
		juju-log "IBM DB2: DB2 not installed"
	fi

}

# Update system configuration after installing DB2
configure_system()
{
	juju-log "IBM DB2: Updating system configuration."
	cd $DB2_INSTALL_PATH/instance
	su - $DB2_USER -c 'db2set DB2COMM=tcpip'

	#Start DB for the DB2_user
	if su - $DB2_USER -c 'ps -eaf|grep -i "\bdb2sysc 0\b" |grep -i '$DB2_USER;
	then
		juju-log "IBM DB2: DB2 already started for the user $DB2_USER"
	else
		su - $DB2_USER -c 'db2start'
		juju-log "IBM DB2: DB2 is started"
	fi

	service_name=`su - $DB2_USER -c 'db2 get dbm cfg|grep -i svce|cut -d"=" -f2'`
	port_num=`grep $service_name /etc/services | cut -d"/" -f1 | cut -f2`
	port_num=`echo $port_num | xargs`
	if su - $DB2_USER -c 'netstat -an | grep '$port_num;
	then
		juju-log "IBM DB2: DB2 started on port $port_num"
	else
		juju-log "IBM DB2: DB2 not started on port $port_num"
		exit 1
	fi

	juju-log "IBM DB2: Updated system configuration."

}

#Configure dftdbpath
configure_dftdbpath()
{
	db2_inst=`is_db2_installed`
	if  [ $db2_inst == "True" ]; then
		cfgdbpath=`config-get dftdbpath`
		#If null, assign the default value
		if [ "cfgdbpath" == "" ]
		then
			cfgdbpath="/tmp"
		fi

		for queue1 in `su - $DB2_USER -c 'db2ilist'`;
		do
			#if db2 is not running don't do anything
			if su - $queue1 -c 'ps -eaf|grep -i "\bdb2sysc 0\b" |grep -i '$queue1;
			then
				su - $queue1 -c 'db2 attach to '$queue1
				dbpath=`su - $queue1 -c 'db2 get dbm cfg|grep -i DFTDBPATH|cut -d"=" -f2'`
				cfgdbpath=`echo $cfgdbpath | xargs`
				dbpath=`echo $dbpath | xargs`
				if  [ "$cfgdbpath" != "$dbpath" ]; then
					if [ -d "$cfgdbpath" ]; then
						su - $queue1 -c 'db2 update dbm cfg using DFTDBPATH '$cfgdbpath
						if [ $? -eq 0 ]; then
							juju-log "IBM DB2: DFTDBPATH changed to $cfgdbpath"
						else
							juju-log "IBM DB2: Unable to set DFTDBPATH to $cfgdbpath"
							exit 0
						fi
					else
						juju-log "IBM DB2: Not a valid path. Please check the path and run the command again"
						exit 0
					fi
				fi
			fi
		done
	else
		juju-log "IBM DB2: DB2 is not installed. Install DB2 to change the dbpath"
		exit 0
	fi
}

# Edit the response file for UID
edit_responsefile()
{
	rsp_file=db2server_temp.rsp
	if [ "$ARCHITECTURE" = "s390x" ]; then
		orgrsp_file=db2server_z.rsp
	else
		orgrsp_file=db2server_ppcle.rsp
	fi
	cp $CHARM_DIR/files/archives/$orgrsp_file $CHARM_DIR/files/archives/$rsp_file

	db2uid=`id $DB2_USER | cut -d"=" -f2 | cut -d"(" -f1`
	sed -i s/db2inst1_uid/$db2uid/g  $CHARM_DIR/files/archives/$rsp_file
	sed -i s/db2inst1_gid/$db2uid/g  $CHARM_DIR/files/archives/$rsp_file
	sed -i s/db2inst1_homedir/$DB2_USER/g $CHARM_DIR/files/archives/$rsp_file
	sed -i s/db2inst1_password/$DB2_USER_PW/g $CHARM_DIR/files/archives/$rsp_file

	db2fenid=`id $DB2_FENCE_USER | cut -d"=" -f2 | cut -d"(" -f1`
	sed -i s/db2fenc1_uid/$db2fenid/g  $CHARM_DIR/files/archives/$rsp_file
	sed -i s/db2fenc1_gid/$db2fenid/g  $CHARM_DIR/files/archives/$rsp_file

	dasuid=`id $DB2_DAS_USER | cut -d"=" -f2 | cut -d"(" -f1`
	sed -i s/das_uid/$dasuid/g $CHARM_DIR/files/archives/$rsp_file
	sed -i s/das_gid/$dasuid/g $CHARM_DIR/files/archives/$rsp_file

}

create_user()
{
	servicename=$1
	user_exists=0
	instance_exists=0
	db2_inst=`is_db2_installed`
	get_cfgusername $servicename
	if  [ $db2_inst == "True" ]; then
		status-set maintenance "Creating DB user and instance"
		#Find the group id for db2inst1 user which is used to install db2 and find all
		#users belonging to this user
		db2inst1grpid=`id $DB2_USER |  cut -d" " -f2`

		#If the user is already available, just log the information.
		for queue2 in `awk -F':' '{ print $1}' /etc/passwd`;
		do
			gid=`id $queue2 |  cut -d" " -f2`
			if [ $gid == $db2inst1grpid ]; then
				if [ $queue2 != "$DB2_USER" ]; then
					if [ $cfgusername == $queue2  ]; then
						juju-log "IBM DB2: User $cfgusername already exists"
						user_exists=1
					fi
				fi
			fi
		done
		

		#If not available, add a new user
		if [ $user_exists -ne 1 ]; then
			juju-log "IBM DB2: Creating New user"
			create_newuser $servicename
			juju-log "IBM DB2: Created New user"
			juju-log $cfgusername
		fi

 		#Set the password if there is no password available in the interface.
		existingpw=$(relation_call --state=db.ready get_db_password $service) || true
		juju-log "IBM DB2: Exisitng password is $existingpw"
		if [ "$existingpw" == "None" ]; then
		#Create a password
			passwd=`pwgen -N 1 15`
			cfgpasswd="$cfgusername$passwd"

			#Set the password for the new user
			juju-log "IBM DB2: Setting the password in new user"
			echo $cfgusername:$cfgpasswd | chpasswd
			if [ $? -ne 0 ]; then
				juju-log "IBM DB2: Setting password for user $cfgusername failed."
				status-set blocked "Setting the password for user failed, Try again"
				exit 0
			fi

			#Send the password to the interface
			juju-log "IBM DB2: Setting password to interface $cfgpasswd"
			relation_call --state=db.ready set_db_password $service $cfgpasswd || true
			password_interface=$(relation_call --state=db.ready get_db_password $servicename) || true
			juju-log "IBM DB2: Getting password from interfce $password_interface"
		fi

		#Creating a new instance for the new user if not existing
		for queue1 in `su - $DB2_USER -c 'db2ilist'`;
		do
			if [ $queue1 == $cfgusername ]; then
				juju-log "IBM DB2: Instance $cfgusername already exists"
				instance_exists=1
			fi
		done
		if [ $instance_exists -ne 1 ]; then
			juju-log "IBM DB2: Creating new instance for the user"
			$DB2_INSTALL_PATH/instance/db2icrt -u $cfgusername $cfgusername
			if [ $? = 0 ]; then
				juju-log "IBM DB2: Created new instance for the user"
			else
				juju-log "IBM DB2: Creation of new instance failed"
				exit 0
			fi
		fi

		#Do the configuration for the new instance created
		new_port=50001
		free_port=0
		is_free=0
		#Check whether the port number is already in etc/services file, and then add it
		cd /etc
		if grep -q "db2c_$cfgusername" services
		then
			juju-log "IBM DB2: Services file already updated"
		else
			while [ $free_port -eq 0 ]
			do
				is_free=`netstat -lnp | grep $new_port | cut -d":" -f2 | cut -d" " -f1`
				if [ "$is_free" != "$new_port" ]; then
					free_port=1
				else
					new_port=$((new_port+1))
				fi
			done
			juju-log "IBM DB2:Updating Services file"
			echo -e "db2c_$cfgusername	$new_port/tcp" >> /etc/services
			su - $cfgusername -c 'set db2instance=$cfgusername'
			su - $cfgusername -c 'db2set DB2COMM=tcpip'
			new_port=`echo $new_port | xargs`
				#port_num_consumer=$new_port
			su - $cfgusername -c 'db2 update dbm cfg using svcename '$new_port
			$DB2_INSTALL_PATH/instance/db2iupdt $cfgusername
		fi
		cd -

		#Start DB for the new user
		if su - $cfgusername -c 'ps -eaf|grep -i "\bdb2sysc 0\b" |grep -i '$cfgusername;
		then
			juju-log "IBM DB2: DB2 already started for the user $cfgusername"
		else
			su - $cfgusername -c 'db2start'
			juju-log "IBM DB2: DB2 is started"
		fi
		#set_state 'db.dbstarted'

	else
		juju-log "IBM DB2: DB2 is not installed. Install DB2 before creating users"
		exit 0

	fi
	status-set active "Completed DB user and instance creation"
}

create_newuser()
{
	servicename=$1
	dbcreated="False"
	#Add the new user
	get_cfgusername $servicename
	useradd -g db2grp1 -G dasadm1 -m $cfgusername
	if [ $? -ne 0 ]; then
		juju-log "IBM DB2: Addition of user $queue2 failed."
		status-set blocked "Addition of new used failed, Try again"
		exit 0
	fi

	#Check whether test DB already exists. If not create it

	dbname=`su - $DB2_USER -c 'db2 list db directory' | grep "Database name" | grep -i "\bTEST\b" | cut -d"=" -f2`
	dbname=`echo $dbname | xargs`
	if [ "$dbname" != "TEST" ]; then
		juju-log "IBM DB2: Creating DB Test"
		su - $DB2_USER -c 'db2 create database test'
		if [ $? -ne 0 ]; then
			juju-log "IBM DB2: DB creation failed"
			status-set blocked "DB creation failed, Try again"
			exit 0
		fi
		juju-log "IBM DB2: Created DB Test"
	fi
	#Connect to the DB and grant data access
	su - $DB2_USER -c 'db2 connect to test;db2 grant dataaccess on database to user '$cfgusername
	if [ $? -ne 0 ]; then
		juju-log "IBM DB2: Grant access failed"
		status-set blocked "Grant dataaccess on database failed, Try again"
		exit 0
	fi
	su - $DB2_USER -c 'db2 connect to test;db2 disconnect test'
	if [ $? -ne 0 ]; then
		juju-log "IBM DB2: Disconnect DB failed"
		status-set blocked "Disconnect DB failed"
		exit 0
	fi

}

create_database()
{
	#isdbcreated=0
	setpassword=0
	dbtocreate=$1
	servicename=$2

	get_cfgusername $servicename
	#create the db for $dbnametocreate if it doesn't exist
	dbname=`su - $cfgusername -c 'db2 list db directory' | grep "Database name" | grep -i "\b$dbtocreate\b" | cut -d"=" -f2`
	dbname=`echo $dbname | xargs`
	dbname=`echo "$dbname" | awk '{ print tolower($1) }'`
	dbtocreate=`echo "$dbtocreate" | awk '{ print tolower($1) }'`
	#check whether dbname is more than 8 chars
	if [ ${#dbtocreate} -gt 8 ]; then
		juju-log "IBM DB2: Failed to create DB $dbtocreate. DB name should be <= 8 chars"
		exit 0 
		#status-set blocked "DB creation for $dbtocreate failed as it contains more than 8 chars"
	else
		if [ "$dbname" != "$dbtocreate" ]; then
			juju-log "IBM DB2: Creating DB $dbtocreate"
			su - $cfgusername -c 'db2 create database '$dbtocreate
			if [ $? -ne 0 ]; then
				juju-log "IBM DB2: DB creation failed"
				status-set blocked "DB creation failed, Try again"
				exit 0
			fi
			juju-log "IBM DB2: Created DB $dbtocreate"

		else
			juju-log "IBM DB2: DB $dbtocreate already exists"
		fi

	fi
}

# Must be called from a 'db.ready' state handler
create_dbs()
{
dbs_created=""
servicename=$1
dbnames=$(relation_call --state=db.ready get_dbnames $servicename) || true

#If dbnames is null, create a default DB as per service name. Else create the DBs requested by the related charm
if [ -z "$dbnames" -o "$dbnames" == "None" ] ; then
	juju-log "IBM DB2: Creating a default DB $remoteunitdbname based on remote service name"
	create_database $remoteunitdbname $servicename
	dbs_created+=$remoteunitdbname
	dbs_created+="."
else
	juju-log "IBM DB2: Creating the DB names as passed by consumer"
	if [ `echo $dbnames | grep -c "," ` -gt 0 ]
	then
		#parse the $dbnames string to get the names of dbs to be created
		count=1
		db_to_create="test"
		while [ "$db_to_create" != "" ]
		do
			db_to_create=`echo $dbnames | cut -d"," -f$count`
			juju-log "IBM DB2:db_to_create is $db_to_create"
			if [ "$db_to_create" != "" ] ; then
				create_database $db_to_create $servicename
				juju-log "Created DB $db_to_create"
				dbs_created+=$db_to_create
				dbs_created+="."
			fi
			((count=count+1))
		done
	else
		create_database $dbnames $servicename
		juju-log "Created DB $dbnames"
		dbs_created+=$dbnames
		dbs_created+="."
	fi
fi
}

remove_sshkeys()
{
	servicename=$1
	#Read the key from the key.txt file and see whether it exists
	if grep -q "$servicename" $TEMP_KEY_FILE
	then
	key=`cat $TEMP_KEY_FILE | grep $servicename | cut -d":" -f2`
	grep -v "$key" $AUTH_KEY_FILE > tempfile.txt || touch tempfile.txt
	mv tempfile.txt $AUTH_KEY_FILE
	if grep -q "$key" $AUTH_KEY_FILE
	then
		juju-log "IBM DB2: SSH Key not deleted from $AUTH_KEY_FILE"
	else
		juju-log "IBM DB2: SSH Key deleted from $AUTH_KEY_FILE"
	fi
	#Remove from the temporary file as well
	grep -v "$servicename:$key" $TEMP_KEY_FILE > tempfile1.txt
	mv tempfile1.txt $TEMP_KEY_FILE
	if grep -q "$key" $TEMP_KEY_FILE
	then
		juju-log "IBM DB2: SSH Key not deleted from $TEMP_KEY_FILE"
	else
		juju-log "IBM DB2: SSH Key deleted from $TEMP_KEY_FILE"
	fi

fi

}


@when 'db.connected'
@when_not 'ibm-db2.installed'
function reset_interface_states(){
	services=$(relation_call --state=db.connected services) || true
	for service in $services; do
		juju-log "IBM DB2: Resetting states for the service $service"
		relation_call --state=db.connected dismiss_sshconfigured $service || true
		relation_call --state=db.connected reset_states $service || true
	done

}


@when_not 'ibm-db2.prereqsinstalled'
function init() {
	if [ "$ARCHITECTURE" != "x86_64" -a "$ARCHITECTURE" != "ppc64le" -a "$ARCHITECTURE" != "s390x" ]; then
		juju-log "IBM DB2: Unsupported platform. IBM DB2 installed with this Charm supports only the x86_64 and POWER LE (ppc64le) and s390x platforms."
		exit 1
	fi
	set -e
	juju-log "IBM DB2: Begin Install."
	status-set maintenance "Installing pre requisites for IBM DB2"
	apt-get update

	if [ "$ARCHITECTURE" == "x86_64" ]; then
		juju-log "IBM DB2: Installing 32 bit libstdc++.so.6"
		apt-get install lib32stdc++6 -y
                apt-get install rpm
		juju-log "IBM DB2: Installed 32 bit libstdc++.so.6"
	fi
	if [ "$ARCHITECTURE" == "s390x" ]; then
                juju-log "IBM DB2: Installing 32 bit libstdc++.so.6"
                apt-get install lib32stdc++6 -y
                juju-log "IBM DB2: Installed 32 bit libstdc++.so.6"

                juju-log "IBM DB2: Installing libaio1 and binutils "
		apt-get install libaio1 
		juju-log "IBM DB2: Installed libaio1 and binutils"
	fi
	if [ "$ARCHITECTURE" == "ppc64le" ]; then
		juju-log "IBM DB2:  Installing libnuma package for POWER LE (ppc64le) platforms."
		apt-get install libnuma-dev -y
                apt-get install rpm
		#Installing XL compiler runtime packages for Linux on Power as this is one of the prerequisite software/configuration requirements
		echo "deb http://public.dhe.ibm.com/software/server/POWER/Linux/rte/xlcpp/le/ubuntu $(lsb_release -s -c) main" | tee -a /etc/apt/sources.list.d/ibm-xlcpp-rte.list
		apt-get update
		apt-get install libxlc -y --force-yes
		juju-log "IBM DB2:  Installed libnuma package for POWER LE (ppc64le) platforms."
	fi

	juju-log "IBM DB2: Creating the user ids for db2 installation"
	groupadd db2grp1
	groupadd dasadm1
	groupadd db2fgrp1

	useradd -g db2grp1 -G dasadm1 -m $DB2_USER
	echo $DB2_USER:$DB2_USER_PW | chpasswd
	useradd -g dasadm1 -G db2grp1 -m $DB2_DAS_USER
	echo $DB2_DAS_USER:$DB2_DAS_USER_PW | chpasswd
	useradd -g db2fgrp1 -m $DB2_FENCE_USER
	echo $DB2_FENCE_USER:$DB2_FENCE_USER_PW | chpasswd

	juju-log "IBM DB2: Created the user ids for db2 installation"
	status-set maintenance "Installed pre requisites for IBM DB2"
	set_state 'ibm-db2.prereqsinstalled'

}

@when 'ibm-db2.prereqsinstalled'
@when_not 'ibm-db2.installed'
function install() {
        # Get the installable db2 resources
        juju-log "IBM DB2: fetching the ibm_db2 _installer resource"
        status-set maintenance "IBM DB2: fetching the ibm_db2 _installer resource"
        db2_pkg_name=`resource-get 'ibm_db2_installer' || echo unavailable`
        ibm_db2_installer_isempty=`file $db2_pkg_name | { grep -q empty && echo "True"; } || echo "False"`
        # If we don't have a package, report blocked status; we can't proceed.
        if [ "$db2_pkg_name" = "unavailable" ]; then
              juju-log "IBM DB2: missing required ibm_db2_installer resource"
              status-set blocked "missing required ibm_db2_installer resource"
              return 0
        fi
                if [ "$ibm_db2_installer_isempty" = "True" ]; then
                        juju-log "IBM DB2: missing required ibm_db2_installer resource, found empty packages"
                        status-set blocked "missing required ibm_db2_installer resource, provide actual ibm_db2_installer resource."
                        return 0
                fi

        ARCHIVE_DIR1=`dirname $db2_pkg_name`
        echo "$ARCHIVE_DIR1"
	juju-log "IBM DB2: License accepted"
	db2_inst=`is_db2_installed`
	if [[ ! -d $ARCHIVE_DIR1 ]]; then
		mkdir $ARCHIVE_DIR1
	fi
	#Install if download succeeded
	if  [ $db2_inst == "False" ]; then
		if [ -f  $ARCHIVE_DIR1/*.gz ]; then

			cd $CHARM_DIR/files/archives
			tar -zxvf $ARCHIVE_DIR1/*.gz
			if [ $? -ne 0 ]; then
				juju-log "IBM DB2: Unable to extract the DB2 package content. Verify whether the package is corrupt."
				status-set maintenance "Unable to extract the DB2 package content"
				#Remove corrupt archive file
				rm -rf $ARCHIVE_DIR1/*.gz
				exit 0
			fi
			if [ "$ARCHITECTURE" = "s390x" ]; then
				rsp_file=db2server.rsp
			else
				rsp_file=db2server_ppcle.rsp
			fi
			edit_responsefile
			cd $CHARM_DIR/files/archives/server*
			juju-log "IBM DB2: Edited the response file"

			juju-log "IBM DB2: Checking etc/hosts file."
			private_address=`unit-get private-address`
			if grep -q "$private_address" /etc/hosts; then
				juju-log "IBM DB2: Host file already updated"
			else
				juju-log "IBM DB2: Updating Host file"
				echo "$private_address `hostname`" >> /etc/hosts
			fi

			juju-log "IBM DB2: Installing db2 package."
			status-set maintenance "Installing IBM DB2"
			./db2setup -r $CHARM_DIR/files/archives/$rsp_file
			juju-log "IBM DB2: Installation of db2 complete."
		fi
	fi
	# Configure system values  for Db2
	configure_system
	#Configure DFTDBPATH
	configure_dftdbpath
	echo "Tempory Key File" > $TEMP_KEY_FILE
	set_state 'ibm-db2.installed'
	status-set active "IBM DB2 Installed"
}

@when 'config.changed.dftdbpath'
function configurevalues(){
	configure_dftdbpath
}

@when 'ibm-db2.installed'
@when_not 'db.connected'
function report_waiting_on_consumers(){
	status-set active "IBM DB2 Ready (waiting for consumers)"
}

@when 'ibm-db2.installed'
@when 'db.connected'
function configure_sshkeys(){
	services=$(relation_call --state=db.connected services)
	for service in $services; do
		key=$(relation_call --state=db.connected get_sshkeys $service) || true
		juju-log "IBM DB2: Received the ssh key $key from the service $service"

		if [ "$key" == "None" ]; then
			juju-log "IBM DB2: No data sent yet"
			continue
		fi

		# Add the key to the authorized_keys file, if it exists
		set +e
		#Create a temp key file if it doesn't exist
		if [ -f $TEMP_KEY_FILE ]; then
			juju-log "IBM DB2: Temp file already exists"
		else
			echo "Temp Key File" > $TEMP_KEY_FILE
		fi
		if [ -f $AUTH_KEY_FILE ]; then
			# Append only if the key is not present
			juju-log "IBM DB2: Updating the key file"
			grep "$key" $AUTH_KEY_FILE > /dev/null
			if [ $? -ne 0 ]; then
				juju-log "IBM DB2: Updating authorized keys file"
				echo "$key" >> $AUTH_KEY_FILE
				echo "$service:$key" >> $TEMP_KEY_FILE
			else
				juju-log "IBM DB2: Key already present"
			fi
		else
			juju-log "IBM DB2: Creating authorized keys file"
			echo "$key" > $AUTH_KEY_FILE
			echo "$service:$key" > $TEMP_KEY_FILE
		fi
		relation_call --state=db.connected set_sshconfigured $service || true
		juju-log "IBM DB2: Completed updating the authorized key"
		set -e
	done
}

@when 'ibm-db2.installed'
@when 'db.ready'
@when 'db.sshconfigured'
function create_consumer_dbs(){
	services=$(relation_call --state=db.ready services)
	for service in $services; do

		key=$(relation_call --state=db.ready get_sshkeys $service) || true
                juju-log "IBM DB2: In create_consumer_dbs function, Received the ssh key $key from the service $service"

                if [ "$key" == "None" ]; then
                        juju-log "IBM DB2: No data sent yet"
                        continue
                fi


		#Create the user for the remote unit connected
		juju-log "IBM DB2: Creating the users for the consumer $service"
		create_user $service
		juju-log "IBM DB2: Create users completed"

		#Create the DBs after parsing
		juju-log "IBM DB2: Creating DBs for the consumer $service"
		create_dbs $service
		juju-log "IBM DB2: Create DBs completed"

		# If we created a new db, send relation data. Otherwise, the db already
		# existed, so there is no need to send the data.
		#if [ $isdbcreated -eq 1 ] ; then
			get_cfgusername $service
			service_name=`su - $cfgusername -c 'db2 get dbm cfg|grep -i svce|cut -d"=" -f2'`
			db2_port=`grep $service_name /etc/services | cut -d"/" -f1 | grep "db2c_$cfgusername" | cut -f2`
			db2_path="/opt/ibm/db2"
			hostname=`unit-get private-address`
			dbusername=$cfgusername
			#dbuserpw=$cfgpasswd
			db2_instance_name=$cfgusername
		
			dbuserpw=$(relation_call --state=db.ready get_db_password $servicename) || true
		        juju-log "IBM DB2: Getting password from interfceand setting it back $dbuserpw"

			juju-log "IBM DB2: Sending the DB details to the consumer $service"
			juju-log "Port number: $db2_port"
			juju-log "Hostname: $hostname"
			juju-log "DBusername: $dbusername DBuserpw: $dbuserpw"
			juju-log "DB Instance name: $db2_instance_name"
			juju-log "Service: $service_name"
			juju-log "DBname: $dbs_created"

			relation_call --state=db.ready set_db_details $service $db2_path $db2_port $hostname $dbusername $dbuserpw $db2_instance_name $dbs_created || true
		#fi

		status-set active "IBM DB2 Ready"
	done
}

@when 'ibm-db2.installed'
@when 'db.departed'
function stop_db2(){
	juju-log "IBM DB2: Stopping the DB when relation is broken"
	services=$(relation_call --state=db.departed services) || true
	for service in $services; do
		juju-log "IBM DB2: Service name is $service"
		get_cfgusername $service
		if su - $cfgusername -c 'ps -eaf|grep -i "\bdb2sysc 0\b" |grep -i '$cfgusername
		then
			su - $cfgusername -c 'db2 force applications all'
			su - $cfgusername -c 'db2stop'
			juju-log "IBM DB2: DB2 stopped for instance $cfgusername"
		else
			juju-log "IBM DB2: DB2 is already stopped for the instance $cfgusername"
		fi
		#Remove the ssh key entry from ssh keys file
		remove_sshkeys $service
		relation_call --state=db.departed dismiss_sshconfigured $service || true
		relation_call --state=db.departed dismiss $service || true
	done
}

reactive_handler_main
